import gab.opencv.*;                                                                   // OpenCV Library importieren
import processing.video.Capture;                   

import org.opencv.core.Mat;
import org.opencv.imgproc.Imgproc;

import processing.serial.Serial;                                                       // Kommunikation mit Arduino

OpenCV opencv;
Capture video;

double[] topf;

void setup() {

  size(640, 480); 
  video = new Capture(this, Capture.list()[0]);
  opencv = new OpenCV(this, width, height);
  video.start();
   

  while (!video.available()) {                                                         // Das Programm läuft schneller als die Kamera. Bis die Kamera eingeschaltet wurde,  
    delay(100);                                                                        // ist schon das Programm ausgeführt. Hierdurch wird dies vermieden.
  }
  video.read();
  opencv.loadImage(video);
  image(video, 0, 0);

  Mat src =opencv.matBGRA;

    
  Mat gray = new Mat();                                                                  //Tief-pass
  Imgproc.cvtColor(src, gray, Imgproc.COLOR_BGR2GRAY);
  Imgproc.medianBlur(gray, gray, 5);
  Mat circles = new Mat();
  Imgproc.HoughCircles(gray, circles, Imgproc.CV_HOUGH_GRADIENT, 1.0, 
    (double)gray.rows()/16,                                                              // Ändern Sie diesen Wert, um Kreise mit unterschiedlichen Abständen zu erkennen
    100.0, 30.0, 160, 180);                                                              // Ändern Sie die letzten beiden Parameter
                                                                                         // (min_radius & max_radius), um größere/kleiner Kreise zu erkennen
  topf = circles.get(0, 0);
}


void draw () {

  if (video.available()) {
    video.read();
    image(video, 0, 0, width, height);                                                    // Das Webcam-Video auf den Bildschirm wiedergeben
    int hueIndex = -1;                                                                    // x-Koordinate des farblich auffäligsten Videopixels
    float hueValue = 0;                                                                   // Wert des farblich auffäligsten Videopixels 
      
                                                                                          // Suche nach dem hellsten Pixel: Für jede Pixelreihe im Videobild und
                                                                                          // für jedes Pixel in der y-ten Zeile den Index jedes Pixels im Video
    video.loadPixels();
    for (int index = 0; index < video.pixels.length; index++) {

      int pixelValue = video.pixels[index];
                                                                                          // Bestimmung des Werts des farblich auffäligsten Videopixels
      float pixelHue = hue(pixelValue);
                                                                                          // Wenn dieser Wert auffäliger als der andere, so wird dieser Wert dieses Pixels gespeichert  
                                                                                          // sowie seine (x, y) Position
      if (pixelHue > hueValue) {
        hueValue = pixelHue;
        hueIndex = index;
      }
    }

    int hueX =  hueIndex % width;
    int hueY =  hueIndex / width;

    noStroke();                                                                       
    smooth();

                                                                                          // Zeichnen Sie einen großen gelben Kreis am farblich auffäligsten Pixel
    fill(255, 204, 0, 128);
    circle(hueX, hueY, 80);

    float circleCenterX = (float)topf[0];
    float circleCenterY = (float)topf[1];
    float circelRadius = (float)topf[2];

    noFill();
    stroke(0);
    circle(circleCenterX, circleCenterY, (circelRadius-10)*2);
   

    if ((pow(circleCenterX-hueX, 2) + pow(circleCenterY-hueY, 2)) < pow((circelRadius-10), 2)){          // Wenn diese Gleichung erfüllt ist, dann ist der Arm innerhalb des Krieses 
      println("Innerhalb");                                                                                      
    }
    
    if ((pow(circleCenterX-hueX, 2) + pow(circleCenterY-hueY, 2)) >= pow((circelRadius-10), 2)) {       // Wenn diese Gleichung erfüllt ist, dann ist der Arm außerhalb des Kreises. 
      println("Außerhalb");
    }
  }
}
